-- get the addon namespace
local addon, ns = ...
local cfg = ns.cfg
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.AddNPC = function(guid)
	if not cfg.npc[guid] then
		cfg.npc[guid]={}
	end
	return cfg.npc[guid]
end

lib.RemoveNPC = function(guid)
	if cfg.Cleave then
		for spell,_ in pairs(cfg.Cleave) do
			if cfg.Cleave[spell].targets[guid] then
				cfg.Cleave[spell].num=cfg.Cleave[spell].num-1
				if cfg.Cleave[spell].num<0 then cfg.Cleave[spell].num=0 end
				if cfg.cleave_current and cfg.cleave_current==spell then
					lib.UpdateCleave(cfg.Cleave[cfg.cleave_current].num)
				end
			end
		end
	end
	if cfg.npc[guid] then
		cfg.npc[guid]=nil
	end
	if cfg.DOT.aura then
		lib.CountDOT()
	end
	return nil
end

lib.CleanNPC = function()
	cfg.npc={}
	lib.ResetCleaveAll()
	if cfg.DOT.aura then
		lib.CountDOT()
	end
	return nil
end



lib.inrange = function(spell)
	if cfg.GUID["target"]==0 then return true end
	if cfg.spells[spell] and cfg.spells[spell].spellbook then 
		if IsSpellInRange(cfg.spells[spell].spellbook,BOOKTYPE_SPELL,"target")==1 then
			if not cfg.spells[spell].inrange then 
				lib.UpdateSpell(spell)
				cfg.Update=true
			end
			cfg.spells[spell].inrange=true
			return true
		else
			if cfg.spells[spell].inrange then
				lib.UpdateSpell(spell)			
				cfg.Update=true
			end
			cfg.spells[spell].inrange=false
			return false
		end
	else
		if IsSpellInRange(spell,"target")==1 then return true end
	end
	return false
end

local onground
lib.GroundUpdate = function()
	onground=cfg.onGround
	if IsFalling() or IsFlying() then
		cfg.onGround=false
	else
		cfg.onGround=true
	end
	
	if onground~=cfg.onGround then
		cfg.Update=true
		return true
	end
	return nil
end

local shape_old={}
lib.UpdateShape = function()
	hedlib.shallowCopy(cfg.shape,shape_old)
	cfg.shape.form=GetShapeshiftForm()
	if cfg.shape.form and cfg.shape.form>0 then
		cfg.shape.spellid=select(5,GetShapeshiftFormInfo(cfg.shape.form))
	else
		cfg.shape.spellid=0
	end
	cfg.shape.id=GetShapeshiftFormID()
	if cfg.shape.id and cfg.shapes[cfg.shape.id] then
		cfg.shape.name=cfg.shapes[cfg.shape.id]
	else 
		cfg.shape.name="human"
	end
	if hedlib.ArrayNotChanged(cfg.shape,shape_old) then
		return nil
	else
		if lib.OnShape then lib.OnShape() end
		cfg.Update=true
		return true
	end
end

local itemID,i
lib.UpdateSet = function()
	cfg.setitems={}
	for itemSET, _ in pairs(cfg.set) do
		cfg.setitems[itemSET]=0
		for _, itemID in pairs(cfg.set[itemSET]) do
			for i=0,19 do
				if cfg.equip[i] and cfg.equip[i]==itemID then
					cfg.setitems[itemSET]=cfg.setitems[itemSET]+1
				end
			end
		end
	end
end

lib.trinket = function(trinket)
	for _, itemID in pairs(cfg.trinket[trinket]) do
		for i=13,14 do
			if cfg.equip[i] and cfg.equip[i]==itemID then
				return true
			end
		end
	end
	return nil
end

lib.SetBonus=function(set)
	if cfg.setitems[set] then
		if cfg.setitems[set]>=4 then
			return 2
		elseif cfg.setitems[set]>=2 then
			return 1
		end
	end
	return 0
end

lib.UpdateEquip=function(slot)
	cfg.equip[slot]=GetInventoryItemID("player", slot)
end

lib.UpdateEquipAll=function()
	for i=0,19 do
		lib.UpdateEquip(i)
	end
	lib.UpdateSet()
end

local cspell
lib.SpellSwitchMode = function()
	if HeddDB.autoswitch and HeddDB.autoswitch=="off" then return end
	if cfg.mode~="dps" then
		for _,cspell in ipairs(cfg.spells_single) do
			if lib.IsLastSpell(cspell) or lib.SpellCasting(cspell) then
				lib.SwitchMode("dps")
				return
			end
		end
	end
	if cfg.mode~="aoe" then
		for _,cspell in ipairs(cfg.spells_aoe) do
			if lib.IsLastSpell(cspell) or lib.SpellCasting(cspell) then
				lib.SwitchMode("aoe")
				return
			end
		end
	end
end

lib.instance=function()
	cfg.instance=select(2,IsInInstance())
	--[[
	arena - Player versus player arena
	none - Not inside an instance
	party - 5-man instance
	pvp - Player versus player battleground
	raid - Raid instance
	]]
	return cfg.instance
end

lib.ininstance=function()
	if cfg.instance=="raid" or cfg.instance=="party" then
		return true 
	else
		return nil
	end
end

lib.raidboss = function()
	if cfg.target=="worldboss" and cfg.instance=="raid" then return true end
	return nil
end

--[[elite - Elite
	normal - Normal
	minus - Small
	rare - Rare
	rareelite - Rare-Elite
	worldboss - World Boss]]
lib.UpdateTarget = function()
	cfg.target=UnitClassification("target")
	if UnitLevel("target")==-1 then cfg.target="worldboss" end
	cfg.GUID["target"]=UnitGUID("target") or 0
	cfg.pvp = UnitPlayerControlled("target")
--	print(cfg.target.." "..cfg.instance)
	return cfg.target
end

lib.GetTargetLVL = function()
	return cfg.tagetlvl[cfg.target] or 2
end

local ppet
lib.UpdatePet = function()
	ppet = cfg.GUID["pet"]
	cfg.GUID["pet"]=UnitGUID("pet") or 0
	if ppet~=cfg.GUID["pet"] then
		cfg.Update=true
	end
end

local mounted
lib.UpdateMount = function()
	mounted=cfg.mounted
	cfg.mounted=IsMounted()
	if cfg.mounted~=mounted then
		cfg.Update=true
		--[[if cfg.mounted then
			print("Mounted!")
		else
			print("Dismounted!")
		end]]
	end
end

lib.hardtarget = function()
	if cfg.target=="normal" or cfg.target=="minus" then return nil end
	return true
end

lib.bosstarget = function()
	if cfg.target=="worldboss" then
		return true
	end
	return nil
end

local health_tmp = {}
lib.UpdateHealth = function(unitID)
	cfg.health[unitID] = cfg.health[unitID] or {}
	hedlib.shallowCopy(cfg.health[unitID],health_tmp)
	cfg.health[unitID].max=UnitHealthMax(unitID) or 0
	cfg.health[unitID].now=UnitHealth(unitID) or 0
	cfg.health[unitID].percent=(cfg.health[unitID].max==0) and 100 or hedlib.round(cfg.health[unitID].now*100/cfg.health[unitID].max,0)
	if not hedlib.ArrayNotChanged(cfg.health[unitID],health_tmp) then
		cfg.Update=true
		return true
	end
end

lib.GetUnitHealth = function(unitID,health)
	health = health or "percent"
	if cfg.GUID[unitID] and cfg.health[unitID] then
		return (cfg.health[unitID][health] or 0)
	end
	return 0
end

local phaste
lib.UpdateHaste = function()
	phaste=cfg.haste
	cfg.haste=GetCombatRatingBonus(CR_HASTE_MELEE)
	if phaste~=cfg.haste then
		if lib.OnHaste then lib.OnHaste() end
		cfg.Update=true
	end
end

lib.HastedCast = function(sec)
	return hedlib.round(sec/(1+cfg.haste/100),2)
end

local enabled, glyphType, glyphTooltipIndex, glyphSpell, icon
lib.HasGlyph = function(id)
	for i=1, NUM_GLYPH_SLOTS do
		enabled, glyphType, glyphTooltipIndex, glyphSpell, icon = GetGlyphSocketInfo(i)
		if glyphSpell==id then return true end
	end
	return nil
end

lib.PrintGlyphs = function()
	for i=1, NUM_GLYPH_SLOTS do
		enabled, glyphType, glyphTooltipIndex, glyphSpell, icon = GetGlyphSocketInfo(i)
		if enabled and glyphSpell then print(i.." "..glyphSpell.." "..select(1,GetSpellInfo(glyphSpell))) end
	end
	return nil
end

local region,text
lib.PrintTooltipLines_helper = function(...)
    for i = 1, select("#", ...) do
        region = select(i, ...)
        if region and region:GetObjectType() == "FontString" then
            text = region:GetText() or " "
			print(region:GetName().." "..text)
        end
    end
end

lib.PrintTooltipLines = function(tooltip)
    lib.PrintTooltipLines_helper(tooltip:GetRegions())
end

lib.PrintSpellTooltip = function(spellid)
	Heddframe.Tooltip:SetOwner(UIParent, 'ANCHOR_NONE')
	Heddframe.Tooltip:SetSpellByID(spellid)
	lib.PrintTooltipLines(Heddframe.Tooltip)
	Heddframe.Tooltip:Hide()
end

lib.IsTextInSpellTooltip = function(spellid,txt)
	Heddframe.Tooltip:SetOwner(UIParent, 'ANCHOR_NONE')
	Heddframe.Tooltip:SetSpellByID(spellid)
	txt = hedlib.IsTextInTooltip(Heddframe.Tooltip,txt)
	Heddframe.Tooltip:Hide()
	return(txt)
end

lib.MainOptionsToggle = function()
	if HeddDB.show=="always" then
		lib.ShowFrame(Heddmain)
		lib.ShowFrame(Heddmain.spells)
	elseif HeddDB.show=="incombat" then
		if cfg.combat then
			lib.ShowFrame(Heddmain)
			lib.ShowFrame(Heddmain.spells)
		else
			lib.HideFrame(Heddmain)
			lib.HideFrame(Heddmain.spells)
		end
	elseif HeddDB.show=="ontarget" then
		if cfg.GUID["target"]==0 and not cfg.combat then
			lib.HideFrame(Heddmain)
			lib.HideFrame(Heddmain.spells)
		else
			lib.ShowFrame(Heddmain)
			lib.ShowFrame(Heddmain.spells)
		end
	end
end

lib.SwitchMode = function(mode)
	if mode then
		if mode~=cfg.mode then
			if mode=="dps" then
				lib.SwitchModeDPS()
			else
				lib.SwitchModeAOE()
			end
		end
	else
		if cfg.mode == "dps" then
			lib.SwitchModeAOE()
		else
			lib.SwitchModeDPS()
		end
	end
end

lib.SwitchModeDPS=function()
	if cfg.plistdps then
		cfg.mode="dps"
		cfg.plist=cfg.plistdps
		cfg.Update=true
	end
end

lib.SwitchModeAOE=function()
	if cfg.plistaoe then
		cfg.mode="aoe"
		cfg.plist=cfg.plistaoe
		cfg.Update=true
	end
end

lib.onclick = function()
	lib.SwitchMode()
end

lib.onclick = function()
	if cfg.noaoe then
		cfg.noaoe=false
		cfg.mode="auto"
	else
		cfg.noaoe=true
		cfg.mode="one"
	end
	Heddmain.text:SetText(cfg.mode)
	cfg.Update=true
end

lib.AddRangeCheck = function(range)
	for key,value in pairs(range) do
		if cfg.spells[value[1]] then
			table.insert(cfg.Range,value[1])
			cfg.RangeColor[value[1]] = value[2] and value[2] or {0,0,0,0}
		end
	end
end

local pinRangeSpell="none"
local inRange=false
lib.rangecheck=function()
	inRange=false
	pinRangeSpell=cfg.inRangeSpell
	cfg.inRangeSpell="none"
	for key,rspell in pairs(cfg.Range) do
		if not inRange then
			if lib.inrange(rspell) then
				inRange=true
				cfg.inRangeSpell=rspell
				if rspell~=pinRangeSpell then
					lib.bdcolor(Heddmain.bd,cfg.RangeColor[cfg.inRangeSpell])
					cfg.Update=true
					--print(rspell)
				end
			end
		else
			lib.inrange(rspell)
		end
	end
	if pinRangeSpell~=cfg.inRangeSpell and cfg.inRangeSpell=="none" then
		lib.bdcolor(Heddmain.bd,{1,0,0,1})
	end
end